"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getMetadata = getMetadata;
exports.getParserOpts = getParserOpts;
exports.locatePluginOpts = locatePluginOpts;

var _babylonLightscript = require("@oigroup/babylon-lightscript");

var parser = _interopRequireWildcard(_babylonLightscript);

var _package = require("../package.json");

var _package2 = _interopRequireDefault(_package);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function getMetadata() {
  return {
    parser,
    getParserOpts,
    name: _package2.default.name,
    version: _package2.default.version,
    options: {
      stdlib: {
        description: "Configure the LightScript standard library"
      },
      patternMatching: {
        description: "Test and branch using `match` keyword",
        valueType: "option",
        options: ["enhanced"],
        defaultValue: "enhanced",
        stage: "1"
      },
      bangCall: {
        description: "Call functions with paren-free syntax using `!`",
        valueType: "boolean",
        stage: "0"
      },
      safeCall: {
        description: "Call a function using `?` to check callability first",
        valueType: "boolean"
      },
      existential: {
        description: "Postfix `?` checks that an expression is not loose-equal to `null`",
        valueType: "boolean"
      },
      enhancedComprehension: {
        description: "Positional comprehensions and `case` syntax for comprehensions",
        valueType: "boolean",
        stage: "0"
      },
      noEnforcedSubscriptIndentation: {
        description: "Do not enforce indentation for subscripts on subsequent lines.",
        valueType: "boolean"
      },
      useRequire: {
        description: "Generate `require` rather than `import` when the compiler introduces a module.",
        valueType: "boolean"
      },
      flippedImports: {
        description: "Allow imports via `import 'path': [specifier]` syntax.",
        valueType: "boolean"
      },
      disableJsx: {
        description: "Don't parse JSX expressions.",
        valueType: "boolean"
      },
      disableFlow: {
        description: "Don't parse Flow type annotations.",
        valueType: "boolean"
      }
    }
  };
}

function getParserOpts(pluginOpts, initialParserOpts) {
  var parserOpts = initialParserOpts || {};
  parserOpts.parser = parser.parse;

  parserOpts.plugins = parserOpts.plugins || [];
  var plugins = parserOpts.plugins;

  plugins.unshift("lightscript");
  plugins.push("objectRestSpread", "asyncFunctions", "asyncGenerators", "classProperties", "classConstructorCall", "decorators", "dynamicImport", "doExpressions");

  if (!(pluginOpts == null ? null : pluginOpts.disableJsx)) plugins.push("jsx");
  if (!(pluginOpts == null ? null : pluginOpts.disableFlow)) plugins.push("flow");
  if (pluginOpts == null ? null : pluginOpts.safeCall) plugins.push("safeCallExpression");
  if (pluginOpts == null ? null : pluginOpts.existential) plugins.push("existentialExpression");
  if (pluginOpts == null ? null : pluginOpts.bangCall) plugins.push("bangCall");
  if (!(pluginOpts == null ? null : pluginOpts.noEnforcedSubscriptIndentation)) plugins.push("enforceSubscriptIndentation");
  if (pluginOpts == null ? null : pluginOpts.flippedImports) plugins.push("flippedImports");
  if (pluginOpts == null ? null : pluginOpts.enhancedComprehension) plugins.push("enhancedComprehension");

  // TODO: watch upstream on pattern matching; default to their syntax when complete
  // patternMatchingVersion = pluginOpts?.patternMatching or "v4"
  // if patternMatchingVersion != "disabled":
  //   plugins.push(`match_${patternMatchingVersion}`)
  plugins.push("match");

  return parserOpts;
} // Attempt to determine whether a post-processed Babel plugin is this
// plugin. This is necessary because Babel doesn't specify the individual
// plugin configuration during manipulateOptions, only the global configuration.
// But we need the local plugin config there to enable parser flags.
// EXTREMELY hackish.


function isMe(entry, thisPlugin) {
  return entry.manipulateOptions === thisPlugin.manipulateOptions;
}

function locatePluginOpts(opts, thisPlugin) {
  var plugins = opts == null ? null : opts.plugins;
  if (!plugins) return {};

  for (var _i2 = 0, _len2 = plugins.length; _i2 < _len2; _i2++) {
    var _plugin = plugins[_i2];

    if (Array.isArray(_plugin)) {
      if (isMe(_plugin[0], thisPlugin)) return _plugin[1];
    } else {
      if (isMe(_plugin, thisPlugin)) return {};
    }
  }

  return {};
}