'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.resetHelpers = resetHelpers;
exports.getHelper = getHelper;
exports.helper = helper;

var _compilerState = require('./compilerState');

var _is = require('./is');

var helpers = {};
var helperPath = null;

function resetHelpers(path) {
  helperPath = path;
  return helpers = {};
}

function insertAfterImports(path, node) {
  for (var _arr2 = path.get("body"), _i2 = 0, _len2 = _arr2.length; _i2 < _len2; _i2++) {
    var _p = _arr2[_i2];

    if (!(0, _is.isa)(_p, "ImportDeclaration")) {
      _p.insertBefore(node);
      // XXX: enabling the below logging command while running the
      // helpers/scope fixture with the following `hasProps` helper:
      //
      // function(obj) {
      //   var i = arguments.length;
      //   if (obj == null) return false;
      //   if (typeof obj !== "object" && typeof obj !== "function") return false;
      //   while (--i > 0) if (!(arguments[i] in obj)) return false;
      //   return true;
      // }
      //
      // ...reveals a severe scope leak bug in Babel wherein the `var i`
      // declaration here gets coalesced with the `const i` declaration
      // from the enclosing scope. This causes a false constant violation to
      // be recorded here, and the es2015-check-constants plugin will
      // then crash on compiled code.
      //
      // I am working around this bug by introducing a new block scope using
      // `for` in the helper, but it should definitely be reported upstream
      // to babel.

      // p.getPrevSibling().scope.dump()
      break;
    }
  }
}

function getHelper(key, makeHelper) {
  var helper = helpers[key];
  if (helper) return helper;

  var functionExpr = makeHelper();
  // babel-template fixup
  if (functionExpr.type === "ExpressionStatement") functionExpr = functionExpr.expression;

  // c/p https://github.com/babel/babel/blob/7.0/packages/babel-core/src/transformation/file/index.js#L206
  var ref = helperPath.scope.generateUidIdentifier(key);

  functionExpr.body._compact = true;
  functionExpr._generated = true;
  functionExpr.id = ref;
  functionExpr.type = "FunctionDeclaration";

  insertAfterImports(helperPath, functionExpr);

  return helpers[key] = ref;
}

var predefinedHelpers = {
  isObject(b) {
    return b.template(`
(function(obj) {
  return obj != null && (typeof obj === "object" || typeof obj === "function")
})
    `)();
  }, hasLength(b) {
    return b.template(`
(function(arr, minLength, maxLength) {
  return(
    arr != null &&
    typeof arr !== "function" &&
    (maxLength === undefined ?
      (minLength ? arr.length >= minLength : arr.length >= 0) :
      (arr.length >= minLength && arr.length <= maxLength)
    )
  )
})
    `)();
  }, hasProps(b) {
    return b.template(`
(function(obj) {
  if (obj == null) return false;
  if (typeof obj !== "object" && typeof obj !== "function") return false;
  for (var i = arguments.length - 1; i > 0; i--)
    if (!(arguments[i] in obj)) return false;
  return true;
})
    `)();
  }, matches(b) {
    return b.template(`
(function(discriminant, atom) {
  return discriminant === atom;
})
    `)();
  } };

function helper(name) {
  var makeHelper = predefinedHelpers[name];
  if (!makeHelper) throw new Error(`no such helper ${name}`);
  return getHelper(name, function () {
    return makeHelper((0, _compilerState.getBabel)());
  });
}